module App.Timer where

import Data.Formatter.Number
import Data.Int
import Prelude

import Control.Monad.Rec.Class (forever)
import Data.Either (Either(..))
import Data.Maybe (Maybe(..))
import Effect.Aff (Milliseconds(..))
import Effect.Aff as Aff
import Effect.Aff.Class (class MonadAff)
import Effect.Class.Console (log)
import Halogen as H
import Halogen.HTML as HH
import Halogen.HTML.Events as HE
import Halogen.HTML.Properties as HP
import Halogen.Subscription as HS

type State =
  { running :: Boolean
  , base :: Int
  , count :: Int
  }

data Action
  = Start
  | Tick
  | Stop
  | Reset
  | Increment
  | Decrement
  | Initialize

component :: forall q i o m. MonadAff m => H.Component q i o m
component =
  H.mkComponent
    { initialState: \_ -> { running: false, base: 0, count: 0 }
    , render
    , eval: H.mkEval H.defaultEval { handleAction = handleAction, initialize = Just Initialize }
    }

render :: forall cs m. State -> H.ComponentHTML Action cs m
render state =
  HH.div
    [ css "card"
    ]
    [ HH.h2_ [ HH.text "PureTimer" ]
    , HH.p_ [ showTime state.count timeFormatter ]
    , HH.button
        [ css "btn-primary"
        , HP.disabled $ state.running || state.base == 0
        , HE.onClick \_ -> Decrement
        ]
        [ HH.text "-1m" ]
    , HH.button
        [ css "btn-primary"
        , HP.disabled state.running
        , HE.onClick \_ -> Increment
        ]
        [ HH.text "+1m" ]
    , HH.div_
        [ HH.button
            [ css "btn-primary"
            , HP.disabled state.running
            , HE.onClick \_ -> Start
            ]
            [ HH.text "Start" ]
        , HH.button
            [ css "btn-primary"
            , HP.enabled state.running
            , HE.onClick \_ -> Stop
            ]
            [ HH.text "Stop" ]
        , HH.p_ [ HH.text $ if state.running then "Running" else "Stopped" ]
        ]
    , HH.div_
        [ HH.button
            [ css "btn-primary"
            , HP.disabled state.running
            , HE.onClick \_ -> Reset
            ]
            [ HH.text "Reset" ]
        ]
    ]

handleAction :: forall cs o m. MonadAff m => Action -> H.HalogenM State Action cs o m Unit
handleAction = case _ of
  Initialize -> do
    _ <- H.subscribe =<< timer Tick
    pure unit
  Increment -> H.modify_ \st -> st { count = st.base + 60, base = st.base + 60 }
  Decrement -> H.modify_ \st -> st { count = st.base - 60, base = st.base - 60 }
  Start -> do
    H.modify_ \st -> st { running = true }
    handleAction Tick
  Tick -> do
    s <- H.get
    if s.count > 0 then
      if s.running then do
        H.put $ { running: s.running, count: s.count - 1, base: s.base }
      else pure unit
    else handleAction Stop
  Stop -> H.modify_ \st -> st { running = false, count = st.count, base = st.base }
  Reset -> H.modify_ \st -> st { count = st.base }

css :: forall r i. String -> HH.IProp (class :: String | r) i
css = HP.class_ <<< HH.ClassName

showTime :: forall cs m. Int -> Either String Formatter -> H.ComponentHTML Action cs m
showTime secs fmt = case fmt of
  Left s -> HH.text s
  Right f -> HH.text $ format f (toNumber (secs / 60)) <> ":" <> format f (toNumber (secs `mod` 60))

timeFormatter :: Either String Formatter
timeFormatter = parseFormatString "00"

timer :: forall m a. MonadAff m => a -> m (HS.Emitter a)
timer val = do
  { emitter, listener } <- H.liftEffect HS.create
  _ <- H.liftAff $ Aff.forkAff $ forever do
    Aff.delay $ Milliseconds 1000.0
    H.liftEffect $ HS.notify listener val
  pure emitter
